from __future__ import unicode_literals

import importlib
import json
import logging

from flask import has_request_context
from flask import request as current_request


def to_json(dikt):
    return json.dumps(dikt)


class LogVersionFilter(logging.Filter):
    def __init__(self, package, key='version'):
        super(LogVersionFilter, self).__init__()
        self.package = package
        self.key = key

    def filter(self, record):
        try:
            pkg = importlib.import_module(self.package)
            setattr(record, self.key, pkg.__version__)
        except:
            pass
        return True


class LogExtraFilter(logging.Filter):
    def __init__(self, **extra):
        super(LogExtraFilter, self).__init__()
        self.extra = extra

    def filter(self, record):
        for k, v in self.extra.iteritems():
            setattr(record, k, v)
        return True


class LogGuruFilter(logging.Filter):
    def __init__(self, default='-'):
        super(LogGuruFilter, self).__init__()
        self.default = default

    def filter(self, record):
        if has_request_context():
            record.guru_id = current_request.guru_id
        else:
            record.guru_id = self.default
        return True


class LogHTTPStatusFilter(logging.Filter):
    def __init__(self, include_codes):
        super(LogHTTPStatusFilter, self).__init__()
        self.include_codes = include_codes

    def filter(self, record):
        if not record.exc_info or not has_request_context():
            return True
        ex = record.exc_info[1]
        if not isinstance(ex, HTTPException):
            return True
        return ex.code in self.include_codes


class RequestLogger(object):

    def __init__(self, logger_name, masked_fields, mask='X'):
        self.logger = logging.getLogger(logger_name)
        self.masked_fields = masked_fields
        self.mask = mask

    def _mask_fields(self, payload):
        if not isinstance(payload, dict):
            return payload
        masked_payload = {}
        for k, v in payload.iteritems():
            if k in self.masked_fields:
                if self.mask is None:
                    continue
                if isinstance(v, basestring):
                    # NOTE: hide masked field length
                    v = self.mask * 8
                else:
                    v = self.mask
            elif isinstance(v, dict):
                v = self._mask_fields(v)
            masked_payload[k] = v
        return masked_payload

    def log(self, response):
        raise NotImplementedError()


class ClientRequestLogger(RequestLogger):

    def _filter_body(self, response):
        return True

    def log(self, response):
        data = {}

        # request
        request = response.request
        data['request'] = {
            'url': request.url,
            'method': request.method,
            }
        data['request']['headers'] = request.headers.items()

        # response
        data['response'] = {
            'status': response.status_code,
            'headers': response.headers.items(),
            }
        if self._filter_body(response):
            if hasattr(response, 'data'):
                data['response']['data'] = response.data
            elif hasattr(response, 'content'):
                data['response']['body'] = response.content

        # guru_id
        if GuruRequestMixin.guru_id_header in request.headers:
            guru_id = request.headers[GuruRequestMixin.guru_id_header]
            data['guru_id'] = guru_id

        raw = to_json(data)
        self.logger.info(raw)


class ServerRequestLogger(RequestLogger):

    def __init__(self,
                 logger_name,
                 masked_fields,
                 mask='X',
                 no_response_body=None,
                 ):
        super(ServerRequestLogger, self).__init__(
            logger_name, masked_fields, mask='X'
        )
        # By default, don't log response body for all 2XX
        # responses. no_response_body can be a mixed list of either status
        # codes or tuples of the form ('METHOD', status_code).
        if no_response_body is None:
            no_response_body = range(200, 300)
        self.no_response_body = set()
        for r in no_response_body:
            if isinstance(r, (int, str)):
                self.no_response_body.add(str(r))
            elif isinstance(r, tuple):
                self.no_response_body.add(tuple(str(p) for p in r))
            else:
                raise TypeError('no_response_body must be a list of ints, '
                                'strs, or tuples')

    def _should_include_response_body(self, status, method):
        # Status is something like '200 OK'
        status_code = status.split()[0]
        method_and_status = (method, status_code)

        return (status_code not in self.no_response_body and
                method_and_status not in self.no_response_body)

    def log(self, response):
        data = {}

        # request
        data['request'] = {
            'url': current_request.url,
            'method': current_request.method,
        }
        try:
            if getattr(current_request, 'payload', None):
                payload = self._mask_fields(current_request.payload)
                data['request']['payload'] = payload
        except Exception:
            pass
        data['request']['headers'] = current_request.headers.to_list(
            charset='utf-8'
        )

        # user_guid
        user_guid = None
        if (hasattr(current_request, 'user') and
                hasattr(current_request.user, 'guid')):
            user_guid = current_request.user.guid
        data['user_guid'] = user_guid

        # guru_id
        data['guru_id'] = current_request.guru_id

        # response
        content_type = response.headers['Content-Type']
        data['response'] = {
            'status': response.status,
            'headers': response.headers.to_list(charset='utf-8')
        }
        if (self._should_include_response_body(response.status,
                                               current_request.method) and
                getattr(response, 'response', None) and
                (content_type in ['application/json', 'application/xml']
                 or content_type.startswith('text/'))):
            body = response.response
            if (isinstance(body, list)
                and all(isinstance(x, basestring) for x in body)):
                body = [c.decode('utf8') for c in body]
                data['response']['body'] = ''.join(body)

        raw = to_json(data)
        self.logger.info(raw)



from __future__ import unicode_literals

import pyramid.threadlocal
from raven import Client as BaseClient, processors
from raven.utils.wsgi import get_current_url, get_environ

from . import Filter as BaseFilter


class Filter(BaseFilter):
    client_cls = PyramidClient


class WSGIRequestProcessor(processors.Processor):

    def get_data(self, data, **kwargs):
        request = pyramid.threadlocal.get_current_request()
        if 'sentry.interfaces.Http' in data or not request:
            return
        environ = request.environ
        headers = dict(request.headers)
        guru_key = getattr(request, 'guru_id_header', None)
        if guru_key and guru_key not in headers:
            headers[guru_key] = getattr(request, 'guru_id', None)

        request_data = {
            'sentry.interfaces.Http': {
                'method': environ.get('REQUEST_METHOD'),
                'url': get_current_url(environ, strip_querystring=True),
                'query_string': environ.get('QUERY_STRING'),
                # TODO
                # 'data': environ.get('wsgi.input'),
                'headers': headers,
                'env': dict(get_environ(environ)),
                'form': dict(request.POST),
            }
        }
        data.update(request_data)


class PyramidClient(BaseClient):

    def __init__(self, dsn=None, **options):
        processors = list(options.pop('processors', []))
        processors.append('chronicle.sentry.BalancedSentryProcessor')
        processors.append('chronicle.sentry.pyramid.WSGIRequestProcessor')
        super(PyramidClient, self).__init__(
            dsn=dsn, processors=processors, **options
        )



from __future__ import absolute_import, unicode_literals

import logging.config
import os

import gunicorn.glogging


class Logger(gunicorn.glogging.Logger):

    def setup(self, cfg):
        if not cfg.logconfig:
            super(Logger, self).setup(cfg)
        else:
            if os.path.exists(cfg.logconfig):
                logconfig = eval(open(cfg.logconfig, 'r').read())
                logging.config.dictConfig(logconfig)
            else:
                raise RuntimeError(
                    "Error: log config '%s' not found" % cfg.logconfig)



from __future__ import unicode_literals

import argparse
import re
import logging


__version__ = '1.0.4'


class LogLevelAction(argparse.Action):

    def __call__(self, parser, namespace, values, option_string=None):
        level = getattr(logging, values.upper())
        setattr(namespace, self.dest, level)


class LogNoiseFilter(logging.Filter):

    excludes = [
        (
            logging.WARNING,
            'newrelic.core.data_collector',
            re.compile('^Data collector is (not contactable|unavailable)\..*')
        ),
        (
            logging.WARNING,
            'newrelic.core.data_collector',
            re.compile('^An unexpected HTTP response was received.*')
        ),
        (
            logging.ERROR,
            'newrelic.core.application',
            re.compile('^Unable to report main transaction metrics.*')
        ),
        (
            logging.ERROR,
            'celery.worker.consumer',
            re.compile('^consumer: Connection to broker lost.*')
        ),
        (
            logging.ERROR,
            'newrelic.core.application',
            re.compile('^Registration of the application.*'),
        ),
        (
            logging.WARN,
            'kombu.mixins',
            re.compile('^Connection to broker lost.*')
        )
    ]

    def filter(self, record):
        for levelno, name, msg_pattern in self.excludes:
            if (record.levelno == levelno and
                        record.name == name and
                        msg_pattern.match(record.message) is not None):
                return False
        return True


def add_logging_opts(parser):
    choices = [
        'DEBUG', 'INFO', 'WARN', 'WARNING', 'ERROR', 'CRITICAL', 'FATAL'
    ]
    parser.add_argument(
        '-l', '--log-level',
        default=logging.WARNING,
        help='Set the logging level',
        choices=[c for c in choices] + [c.lower() for c in choices],
        action=LogLevelAction,
    )
    parser.add_argument(
        '--enable-syslog', action='store_true', default=False)
    parser.add_argument(
        '--disable-stderrlog', action='store_true', default=False)
    parser.add_argument(
        '--enable-emaillog', action='store_true', default=False)
    parser.add_argument(
        '--enable-sentrylog', action='store_true', default=False)


class BalancedParser(argparse.ArgumentParser):

    def __init__(self, *a, **kw):
        super(BalancedParser, self).__init__(*a, **kw)
        add_logging_opts(self)

    def configure_logging(self, logging_fn, args=None):
        args = args or self.parse_args()
        logging_fn(
            enable_console=not args.disable_stderrlog,
            enable_syslog=args.enable_syslog,
            enable_email=args.enable_emaillog,
            enable_sentry=args.enable_sentrylog,
            level=args.log_level,
        )


class NoExcTextFormatter(logging.Formatter):
    """
    Formats records identically to logging.Formatter but does *not* append
    record.exc_text.
    """

    def format(self, record):
        record.message = record.getMessage()
        if self.usesTime():
            record.asctime = self.formatTime(record, self.datefmt)
        s = self._fmt % record.__dict__
        return s


class ExtraFilter(logging.Filter):
    """
    Attaches extra fields to log records if not already there, typically used
    to ensure that a log record has certain fields by default once it reaches
    a handler, e.g.:

    .. code:: python

        logging.config.dictConfig({
        ...
        'filters': {
            'defaults': {
                '()': 'sterling.log.ExtraFilter',
                'guru_id': '-',
            },
        ...
        })

    Adding a 'defaults' filter to a handler ensures guru_id will be defaulted
    to  '-' if not otherwise set.
    """

    def __init__(self, **extra):
        super(ExtraFilter, self).__init__()
        self.extra = extra

    def filter(self, record):
        for k, v in self.extra.iteritems():
            if not hasattr(record, k):
                setattr(record, k, v)
        return True



from __future__ import unicode_literals
import logging

from raven import processors


class Formatter(logging.Formatter):
    """
    Formats records identically to `logging.Formatter` but does *not* append
    `record.exc_text`.
    """

    def format(self, record):
        record.message = record.getMessage()
        if self.usesTime():
            record.asctime = self.formatTime(record, self.datefmt)
        s = self._fmt % record.__dict__
        return s


class Filter(logging.Filter):
    """
    Captures, encodes and attaches exception as a sentry message and attaches
    it the logging record.

    You can then reference the encoded sentry message in a loggig formatter:

        %(sentry)s

    """

    client_cls = None

    def __init__(self, project, **options):
        self.extra = options.pop('extra', [])
        self.client = self.client_cls(project=project, **options)

    def filter(self, record):
        if not record.exc_info:
            # ignore
            return False
        try:
            extra = dict(
                (k, getattr(record, k))
                for k in self.extra if hasattr(record, k)
            )
            msg = self.client.build_msg(
                'raven.events.Exception',
                exc_info=record.exc_info,
                extra=extra,
            )
            encoded = self.client.encode(msg)
            record.sentry = encoded
        except:
            # ignore
            return False
        return True


class SanitizeProcessor(processors.SanitizePasswordsProcessor):

    FIELDS = frozenset([
        'password', 'secret', 'card_number', 'account_number',
        'security_code', 'ssn', 'ssn_last_4', 'ssn_last4', 'tax_id',
    ])


class BalancedSentryProcessor(SanitizeProcessor):

    pass



from __future__ import unicode_literals

import os
import re
import setuptools


version = (
    re
        .compile(r".*__version__ = '(.*?)'", re.S)
        .match(open('chronicle/__init__.py').read())
        .group(1)
)

packages = [
    str(s) for s in
    setuptools.find_packages('.', exclude=('tests', 'tests.*'))
]

here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, 'README.md')) as f:
    README = f.read()

with open(os.path.join(here, 'CHANGES.txt')) as f:
    CHANGES = f.read()

requires = []

extras_require = {
    'tests': [
        'blumpkin>=0.4.0,<0.5.0',
        'ipdb',
        'webtest',
    ],
    'sentry': [
        'raven >=3.5,<3.6',
    ],
    'flask': [
        'flask',
    ],
    'pyramid': [
        'pyramid',
    ],
    'gunicorn': [
        'gunicorn',
    ]
}

scripts = []


setuptools.setup(
    name='chronicle',
    version=version,
    description='logging utils',
    long_description=README + '\n\n' + CHANGES,
    classifiers=[
        "Programming Language :: Python",
    ],
    author='Balanced',
    author_email='dev+bob@balancedpayments.com',
    packages=packages,
    include_package_data=True,
    zip_safe=False,
    scripts=scripts,
    install_requires=requires,
    extras_require=extras_require,
    tests_require=extras_require['tests'],
    test_suite='nose.collector',
)
